var OpenLayers = OpenLayers;
var ol = ol;
var Buildings = Buildings;
var google = google;
var $ = $;


function KULMAP() {
    "use strict";
    var that = this;
    
    this.bounds = undefined;
    this.map = undefined;
    this.buildings = {};
    this.currentBuildingLayer = undefined;
    this.visible = undefined;
    this.selectedFeatures = undefined;
    this.select = undefined;
    this.features = {};
    this.buildingOverlay = undefined;
    this.currentZoom = 14;
    this.featureVisibleStyle = new ol.style.Style({
        stroke: new ol.style.Stroke({
            color: 'rgba(0, 0, 0, 0.5)',
            width: 1
        }),
        fill: new ol.style.Fill({
            color: 'rgba(100, 100, 100, 0.3)'
        })
    });
    this.featureNotVisibleStyle = new ol.style.Style({
        stroke: new ol.style.Stroke({
            color: 'rgba(0, 0, 0, 0)',
            width: 1
        }),
        fill: new ol.style.Fill({
            color: 'rgba(100, 100, 100, 0)'
        })
    });
    
    /**
    * Initialize the map
    */
    this.init = function () {
        // Setup view
        var view = new ol.View2D(),
            mapCanvas = document.getElementById('olmap'),
            i,
            that = this,
            pinch = true;
        view.setCenter(ol.proj.transform([4.6931, 50.8734], 'EPSG:4326', 'EPSG:3857'));
        view.setZoom(14);
        
        // Load building data and make buildingOverlay features
        var featureCollection = new ol.Collection();
        var geojson = new ol.source.GeoJSON({
            projection: 'EPSG:3857',
            url: './data/kulmap.geojson'
        });
        geojson.on('addfeature', function (event) {
            var props = event.feature.getProperties();
            if(props.floors !== undefined && props.lonlat !== undefined) {
                var building = new Building(props.name, props.floors.split(","), props.lonlat.split(","),props.tags.split(",").join(" "), props.rooms);
                that.addBuildingLayers(building);
                addBuildingToSearch(building);
                Buildings[props.name] = building;
                that.features[props.name] = event.feature;
                featureCollection.push(event.feature);
                event.feature.setStyle(that.featureNotVisibleStyle);
            }
        });
        
        var vectorPolygons = new ol.layer.Vector({
            source: geojson
        });
        
        // Init buildingSelect interaction
        this.select = new ol.interaction.Select({
            style: this.featureNotVisibleStyle,
            toggleCondition: ol.events.condition.never
        });
        this.selectedFeatures = this.select.getFeatures(); 
        this.selectedFeatures.on('add', this.selectBuilding); 
        this.selectedFeatures.on('remove', this.clearBuildingSelection);
        
        // Init map
        this.map = new ol.Map({
            interactions: ol.interaction.defaults().extend([this.select]),
            controls: [new ol.control.Zoom()],
            layers: [
                new ol.layer.Tile({
                    source: new ol.source.MapQuest({layer: 'osm'})
                }),
                vectorPolygons
            ],
            renderer: 'canvas',
            target: mapCanvas,
            view: view,
            ol3Logo: false
        });
        
        // Listen to moveEnd event
        this.map.on('moveend', this.moveEndHandler);
        
//        var zoomLevel = 0;
//        $(window).on('touchstart', function(evt) {
//            zoomLevel = that.map.getView().getZoom();
//        });
//        
//        $(window).on('touchend', function(evt) {
//            if(zoomLevel === 0) {
//                return false;
//            }
//            var newZoomLevel = that.map.getView().getZoom();
//            if(newZoomLevel > zoomLevel) {
//                Analytics.trackEvent('Map','ZoomIn from ' + zoomLevel + ' to ' + newZoomLevel,'PinchZoomIn');
//            } else if (newZoomLevel < zoomLevel) {
//                Analytics.trackEvent('Map','ZoomOut from ' + zoomLevel + ' to ' + newZoomLevel,'PinchZoomOut');
//            }
//            that.currentZoom = newZoomLevel;
//            zoomLevel = 0;
//        });
        
        this.buildingOverlay = new ol.FeatureOverlay({
            features: featureCollection,
            map: this.map,
            style: this.featureVisibleStyle
        });
        
        document.getElementsByName("ZoomOut")[0].addEventListener("click",function(evt){
            pinch = false;
            var newZoom = that.map.getView().getZoom();
            that.currentZoom = newZoom;
            Analytics.trackEvent('Map','ZoomOut from ' + (newZoom + 1) + ' to '+ newZoom,'ZoomButtonOut');
        });
        document.getElementsByName("ZoomIn")[0].addEventListener("click",function(evt){
            pinch = false;
            var newZoom = that.map.getView().getZoom();
            that.currentZoom = newZoom;
            Analytics.trackEvent('Map','ZoomIn from ' + (newZoom - 1) + ' to '+newZoom,'ZoomButtonIn');
        });
        
        
    };
    
    /**
    * Switches current building to one floor down.
    */
    this.switchToFloorDown = function (amount) {
        Analytics.trackEvent("Map","Switched floor "+this.visible+" to " + amount + " down in building "+this.currentBuilding["name"],"FloorsDown");
        this.currentBuilding[this.visible].setVisible(false);
        this.visible = this.currentBuilding.floors[this.currentBuilding.floors.indexOf(this.visible.toString()) - amount];
        this.currentBuilding[this.visible].setVisible(true);
    };
    
    /**
    * Switches current building to one floor up.
    */
    this.switchToFloorUp = function (amount) {
        Analytics.trackEvent("Map","Switched floor "+this.visible+" to " + amount + " up in building "+this.currentBuilding["name"],"FloorsUp");
        this.currentBuilding[this.visible].setVisible(false);
        this.visible = this.currentBuilding.floors[this.currentBuilding.floors.indexOf(this.visible.toString()) + amount];
        this.currentBuilding[this.visible].setVisible(true);
    };
    
    /**
    * Adds building layers of given building to the map, visibility is set to false by default.
    */
    this.addBuildingLayers = function (building) {
        var building,
            name,
            i,
            floors = {};
        
        for (i = 0; i < building.floorList.length; i += 1) {
            floors[building.floorList[i]] = (new ol.layer.Tile({
                preload: Infinity,
                source: new ol.source.XYZ({
                    url: "http://www.gelekeizer.be/" + building.name.split(' ')[0] + "/" + building.floorList[i] + "/{z}/{x}/{y}.png",
                    maxZoom: 22
                }),
                visible: false,
                maxResolution: 1.194328566955879
            }));
        }
        this.buildings[building.name] = floors;
    };
    
    /**
    * Go to location given by longitude,latitude and zoom
    */
    this.goToLocation = function (lonlat, zoom) {
        var zm = ol.animation.zoom({
            resolution: this.map.getView().getResolution(),
            duration: 3500
        }),
            pan = ol.animation.pan({
                source: this.map.getView().getCenter()
            }),
            i;
        this.map.beforeRender(pan,zm);
        this.map.getView().setCenter(ol.proj.transform(lonlat, 'EPSG:4326', 'EPSG:3857'));
        this.map.getView().setZoom(zoom);
    };
    
    /**
    * When a search result is clicked, a new building must be selected.
    */
    this.searchDone = function (buildingName) {
        if(this.selectedFeatures.getLength() !== 0) {
            this.selectedFeatures.pop();
        }
        this.selectedFeatures.push(this.features[buildingName]);
    };
    
    /**
    * Handles the end of a move (zoom, pan, rotate, ...)
    */
    this.moveEndHandler = function (evt) {
        if(that.selectedFeatures !== undefined && that.selectedFeatures.getArray().length > 0) {
            if(that.map.getView().getZoom() < 18) { // When zoomed out far enough, no building is selected anymore.
                    that.selectedFeatures.pop();
            } else if(that.map.getView().getZoom() >= 18) { // When zoomed in enough, the floorbuttons are shown if building is in view.
                var inView = that.buildingInView(that.selectedFeatures.getArray()[0].get('name'));
                if($('#buttonContainer').hasClass('ui-screen-hidden') && inView){
                    $('#buttonContainer').removeClass('ui-screen-hidden');
                } else if(!inView && !$('#buttonContainer').hasClass('ui-screen-hidden')) {
                    $('#buttonContainer').addClass('ui-screen-hidden');
                }
            }
        }
        if(that.currentZoom !== that.map.getView().getZoom()) {
            var newZoomLevel = that.map.getView().getZoom();
            if(newZoomLevel > that.currentZoom) {
                Analytics.trackEvent('Map','ZoomIn from ' + that.currentZoom + ' to ' + newZoomLevel,'ZoomIn');
            } else if (newZoomLevel < that.currentZoom) {
                Analytics.trackEvent('Map','ZoomOut from ' + that.currentZoom + ' to ' + newZoomLevel,'ZoomOut');
            }
            that.currentZoom = newZoomLevel;
        }
    }
    
    /**
    * Selects a building: add building layers to map, load floor buttons and jump to building
    */
    this.selectBuilding = function (event) {
        that.buildingOverlay.removeFeature(event.element);
        var props = event.element.getProperties();
        Analytics.trackEvent("Map","Selected building "+ props.name,"BuildingFeaturesSelected");
        var floorNbs = props.floors.split(",");
        var i;
        for(i = 0; i < floorNbs.length; i += 1){
            that.map.addLayer(that.buildings[props.name][floorNbs[i]]);
        }
        that.currentBuilding = that.buildings[props.name];
        that.currentBuilding["floors"] = floorNbs;
        if(that.visible !== undefined && that.visible.indexOf("search_") !== -1) {
            that.visible = that.visible.slice(7);
        }
        if(that.visible === "" || that.visible === undefined){
            that.currentBuilding["0"].setVisible(true);
            that.visible = "0";
        } else {
            that.currentBuilding[that.visible].setVisible(true);
        }
        that.currentBuilding["name"] = props.name;
        $('#buttonContainer').removeClass('ui-screen-hidden');
        loadFloors(floorNbs, that.visible);
        if(that.map.getView().getZoom() < 18) {
            that.goToLocation(props.lonlat.split(',').map(function(item) {
                return parseFloat(item, 10);
            }), 18);
        }
        $('#filter-input').val(props.name).trigger('change');
        $('#searching').addClass('ui-screen-hidden');
    }
    
    /**
    * Selects building selection: remove floorlayers and hide floorbuttons
    */
    this.clearBuildingSelection = function (event) {
        that.buildingOverlay.addFeature(event.element);
        if(that.visible.indexOf("search_") !== -1) {
            that.visible = that.visible.slice(7);
        } else {
            that.visible = "";
        }
        var props = event.element.getProperties();
        Analytics.trackEvent("Map","UnSelected building "+ props.name,"BuildingFeaturesUnSelected");
        var floorNbs = props.floors.split(",");
        var i;
        for(i = 0; i < floorNbs.length; i += 1){
            that.buildings[props.name][floorNbs[i]].setVisible(false);
            that.map.removeLayer(that.buildings[props.name][floorNbs[i]]);
        }
        if(that.currentBuilding === that.buildings[props.name]) {
            that.currentBuilding = undefined;
        }
        $('#buttonContainer').addClass('ui-screen-hidden');
        loadFloors(floorNbs, "0");
    }
    
    /**
    * Checks if a building is in the view.
    */
    this.buildingInView = function (buildingName) {
        var featCoordinates = that.features[buildingName].getGeometry().getCoordinates()[0];
        var filterNeg = function (el) {
            return el < 0;
        };
        var allBig = featCoordinates.filter(function (coord) {
            var pixl = that.map.getPixelFromCoordinate(coord);
            return pixl[0] > that.map.getSize()[0] || pixl[1] > that.map.getSize()[1];
        }).length === featCoordinates.length;
        var allNeg = featCoordinates.filter(function (coord) {
            return that.map.getPixelFromCoordinate(coord).filter(filterNeg).length > 0;
        }).length === featCoordinates.length;
        return !allBig && !allNeg;
    }
}